﻿init python:
    import random
    import time
    random.seed(time.perf_counter_ns())

    renpy.music.register_channel("sfx1", "sfx", True)

    def get_neghburs(board, x, y):
        tiles = [
            [-1, -1], [0, -1], [1, -1],
            [-1, 0], [1, 0],
            [-1, 1], [0, 1], [1, 1],
        ]
        new_tiles = []
        for i in tiles:
            t = [y + i[0], x + i[1]]
            if t[0] < 0 or t[1] < 0 or t[0] > len(board)-1 or t[1] > len(board[0])-1:
                pass
            else:
                new_tiles.append(t)
        return new_tiles


    class minesweeper_tile:
        def __init__(self, icon):
            self.icon = icon
            self. revealed = False
            self. flagged = False
        
    class minesweeper_class:
        def __init__(self):
            self.x = 8
            self.y = 8
            self.mines = 8
            self.map = []
            self.restart()
            self.collaps_list = []
            self.status = "playing"
            self.flags = 0
            
        def restart(self):
            land = []
            for i in range(self.x*self.y):
                if i < self.mines:
                    land.append(minesweeper_tile("mines_mine_2"))
                else:
                    land.append(minesweeper_tile(0))
            random.shuffle(land)
            chunked_land = []
            for i in range(0, len(land), self.x):
                chunked_land.append(land[i:i+self.x])
            self.map = chunked_land
            self.calculate_bombs()
            self.status = "playing"
            self.flags = 0
            renpy.play("minesweeper/turn.ogg", "sound")
        def calculate_bombs(self):
            for y,j in enumerate(self.map):
                for x,i in enumerate(j):
                    if i.icon == "mines_mine_2":
                        neighburs = get_neghburs(self.map, x, y)
                        for n in neighburs:
                            if self.map[n[0]][n[1]].icon != "mines_mine_2":
                                self.map[n[0]][n[1]].icon += 1
        def clicked(self, x, y):
            if not self.status == "playing":
                return
                
            target = self.map[y][x]
            if target.revealed or target.flagged:
                return
            else:
                target.revealed = True
                renpy.play("minesweeper/turn.ogg", "sound")

            if target.icon == "mines_mine_2":
                target.icon = "mines_explode"
                Show("minesweeper_revealer", g = self)()
                self.status = "lose"
                renpy.play("minesweeper/explode.ogg", "sfx1")
                return
            elif target.icon == 0:
                self.collaps_list.append([y,x])
                Show("minesweeper_collapser", g = self)()
            self.win_check()
        def collaps(self):
            if len(self.collaps_list):
                spot = self.collaps_list.pop(0)
                neighburs = get_neghburs(self.map, spot[1], spot[0])
                for i in neighburs:
                    tile = self.map[i[0]][i[1]]
                    if not tile.revealed:
                        if tile.icon == 0:
                            if not i in self.collaps_list:
                                self.collaps_list.append(i)
                    tile.revealed = True
            else:
                Hide("minesweeper_collapser")()
                self.win_check()
        def reveal_all(self):
            for j in self.map:
                for i in j:
                    i.revealed = True
            Hide("minesweeper_revealer")()
        def flag(self, x, y):
            target = self.map[y][x]
            if not target.revealed:
                if target.flagged:
                    self.flags -= 1
                    target.flagged = False
                else:
                    self.flags += 1
                    target.flagged = True
                renpy.play("minesweeper/flag.ogg", "sound")

        def win_check(self):
            for j in self.map:
                for i in j:
                    if not i.icon == "mines_mine_2":
                        if not i.revealed:
                            return
            else:
                self.status = "won"
                renpy.play("minesweeper/win.ogg", "sfx1")
                # Show("minesweeper_revealer", g = self)()
        def change_width(self, amount):
            self.x += amount
            if self.x < 2:
                self.x = 2
            renpy.play("minesweeper/turn.ogg", "sound")
        def change_height(self, amount):
            self.y += amount
            if self.y < 2:
                self.y = 2
            renpy.play("minesweeper/turn.ogg", "sound")
        def change_mines(self, amount):
            self.mines += amount
            if self.mines < 1:
                self.mines = 1
            renpy.play("minesweeper/turn.ogg", "sound")

transform zoom(z, n=False):
    nearest n
    zoom z
image mines_mine = Crop((0,0,16,16), "minesweeper/minesweeper.png")
image mines_mine_2 = Crop((16,0,16,16), "minesweeper/minesweeper.png")
image mines_flag = Crop((32,0,16,16), "minesweeper/minesweeper.png")
image mines_explode = Crop((48,0,16,16), "minesweeper/minesweeper.png")

image mines_1 = Crop((0,16,16,16), "minesweeper/minesweeper.png")
image mines_2 = Crop((16,16,16,16), "minesweeper/minesweeper.png")
image mines_3 = Crop((32,16,16,16), "minesweeper/minesweeper.png")
image mines_4 = Crop((48,16,16,16), "minesweeper/minesweeper.png")
image mines_5 = Crop((0,32,16,16), "minesweeper/minesweeper.png")
image mines_6 = Crop((16,32,16,16), "minesweeper/minesweeper.png")
image mines_7 = Crop((32,32,16,16), "minesweeper/minesweeper.png")
image mines_8 = Crop((48,32,16,16), "minesweeper/minesweeper.png")

image mines_panel = Crop((32,48,16,16), "minesweeper/minesweeper.png")
image mines_frame = Crop((48,48,16,16), "minesweeper/minesweeper.png")

default minesweeper_ins = minesweeper_class()
screen minesweeper_screen(g = minesweeper_ins):
    modal True
    frame:
        at zoom(4,True)
        background Frame("mines_frame", 3,3) padding 3,3 align .5,.5
        vbox spacing 0:
            for y,j in enumerate(g.map):
                hbox spacing 0:
                    for x,i in enumerate(j):
                        button:
                            xysize 16,16 padding 0,0
                            if i.revealed:
                                if i.icon == 0:
                                    pass
                                elif i.icon in [1,2,3,4,5,6,7,8]:
                                    add "mines_[i.icon]"
                                else:
                                    add i.icon
                            else:
                                add "mines_panel"
                            if i.flagged:
                                add "mines_flag"

                            action Function(g.clicked, x, y)
                            alternate Function(g.flag, x, y)
    frame align .5,.1:
        style_prefix "minesweeper"
        background Frame("mines_frame", 4,4) padding 20,20
        if g.status == "playing":
            text "[g.flags] of [g.mines] mines are found."
        elif g.status == "won":
            text "You've won, care to play again?"
        elif g.status == "lose":
            text "Haha, you're aloser, always been a failur."

    if not g.status == "playing":
        drag offset -100,0 align 1.0,.5:
            style_prefix "minesweeper"
            frame:
                background Frame("mines_frame", 4,4) padding 20,20
                vbox spacing 20:
                    vbox:
                        text "Width"
                        hbox:
                            button:
                                text "<"
                                action Function(g.change_width, -1)
                            text str(g.x)
                            button:
                                text ">"
                                action Function(g.change_width, 1)
                    vbox:
                        text "Height"
                        hbox:
                            button:
                                text "<"
                                action Function(g.change_height, -1)
                            text str(g.y)
                            button:
                                text ">"
                                action Function(g.change_height, 1)
                    vbox:
                        text "Mines"
                        hbox:
                            button:
                                text "<"
                                action Function(g.change_mines, -1)
                            text str(g.mines)
                            button:
                                text ">"
                                action Function(g.change_mines, 1)
                    button:
                        text "Restart"
                        action Function(g.restart)

style minesweeper_text:
    size 40
    align (.5,.5)
    font "fonts/chary___.ttf"
style minesweeper_hbox:
    align (.5,.5)
    spacing 20
style minesweeper_vbox:
    align (.5,.5)
style minesweeper_button:
    align (.5,.5)
    background Frame("mines_frame", 4,4)
    padding (20,10)

screen minesweeper_collapser(g):
    modal True
    timer .02 repeat True action Function(g.collaps)
screen minesweeper_revealer(g):
    timer .6 repeat True action Function(g.reveal_all)


label minesweeper_example:
    show bg
    call screen minesweeper_screen



